do
    table = table or {}
    table.tools = table.tools or {}
    local Observer = table.tools.Observer

    --- version 2 ---
    ---@class Stopwatch
    Clock = {}

    ---@public
    function Clock:start()
        self.metronome:start()
        return self
    end

    ---@public
    ---@param callback function
    function Clock:add_tick_event(callback)
        self.tick_event:subscribe(callback)
    end

    ---@public
    ---@param callback function
    function Clock:remove_tick_event(callback)
        self.tick_event:unsubscribe(callback)
    end

    ---@public
    ---@param cls Stopwatch
    function Clock.create(cls, period)
        local obj = setmetatable({}, cls.__meta) ---@type Stopwatch
        obj.tick_event = Observer() ---@type Observer
        obj.elapsed_time = 0
        obj.period = period or 0.01
        obj.tick_callback = function()
            obj.elapsed_time = tonumber(string.format(
                    "%.3f",
                    obj.elapsed_time + obj.period + 0.00001
            ))
            obj.tick_event:publish(obj.elapsed_time)
        end
        obj.metronome = Metronome(obj.period)
        obj.metronome:add_tick_event(obj.tick_callback)
        return obj
    end

    Clock.__meta = {
        __index = Clock
    }

    setmetatable(Clock, {
        __call = Clock.create
    })
end