do
    --- version 2 ---
    ---@class RecycledTimer
    ---@field __recycle_pull set<RecycledTimer>
    ---@field __timer_dict set<RecycledTimer>
    ---@field __meta table
    ---@field running_flag boolean
    ---@field periodic_flag boolean
    ---@field recycled_flag boolean
    ---@field callback function
    ---@field timer_handle handle
    RecycledTimer = {}

    ---@param timeout number
    ---@param callback function
    function RecycledTimer:start(timeout, callback,...)
        self.cache = {...}
        self.callback = callback
        TimerStart(self.timer_handle, 0.00, false, self.timer_call)
    end

    function RecycledTimer:recycle()
        self.recycled_flag = true
        self.periodic_flag = false
        self.callback = nil
        setmetatable(self.cache, self.weak_meta)
        self.cache = nil
        self.__recycle_pull:add(self)
    end

    ---@return boolean
    function RecycledTimer:is_periodic()
        return self.periodic_flag
    end

    ---@return boolean
    function RecycledTimer:is_recycled()
        return self.recycled_flag
    end

    -------------- class methods --------------------

    ---@public
    ---@param cls RecycledTimer
    function RecycledTimer.get_expired(cls)
        return cls:get(GetExpiredTimer())
    end

    ---@private
    ---@param cls RecycledTimer
    ---@param timer_handle handle
    function RecycledTimer.get(cls, timer_handle)
        local handle_id = GetHandleId(timer_handle)
        return cls.__timer_dict:get(handle_id)
    end

    ---@private
    ---@param cls RecycledTimer
    ---@param obj RecycledTimer
    function RecycledTimer.add(cls, obj)
        local handle_id = GetHandleId(obj.timer_handle)
        cls.__timer_dict:set(handle_id, obj)
    end

    ---@private
    ---@param cls RecycledTimer
    function RecycledTimer.get_recycled(cls)
        local obj = cls.__recycle_pull:get_last()
        if obj == nil then return nil end
        cls.__recycle_pull:remove(obj)
        return obj
    end

    --- factory for recycle
    ---@public
    ---@param cls RecycledTimer
    function RecycledTimer.create(cls)
        local obj = cls:get_recycled()
        if obj == nil then
            obj = cls:__new()
        end
        return obj
    end

    ---@private
    ---@param cls RecycledTimer
    function RecycledTimer.__new(cls)
        local obj = setmetatable({}, cls.__meta) ---@type RecycledTimer
        obj.timer_handle = CreateTimer()
        obj.timer_call = function()
            obj.running_flag = false
            if #obj.cache == 0 then
                obj.callback()
            else
                obj.callback(table.unpack(obj.cache))
            end
            obj:recycle()
            --obj = nil
        end
        obj.periodic_flag = nil
        obj.recycled_flag = false
        cls:add(obj)
        cls.count = cls.count + 1
        print("я создался", cls.count)
        return obj
    end

    ---@public
    ---@param cls RecycledTimer
    function RecycledTimer.init(cls)
        cls.count = 0
        cls.__recycle_pull = set()
        cls.__timer_dict = dict()
        cls.weak_meta = {__mode = 'kv'}
        cls.init = function(_cls) return _cls end
        return cls
    end

    RecycledTimer.__meta = {
        __index = RecycledTimer
    }

    setmetatable(RecycledTimer, {
        ---@param cls RecycledTimer
        __call = function(cls,...)
            getmetatable(cls).__call = cls.create
            cls:init()
            return cls:create(...)
        end
    })
end